// Fitting MGLMM Simplex 
#include <TMB.hpp>
template<class Type>
Type objective_function<Type>::operator() ()
{
  using namespace density;
  DATA_VECTOR(Y1);
  DATA_VECTOR(Y2);
  DATA_VECTOR(Y3);
  DATA_VECTOR(Y4);
  DATA_VECTOR(Y5);
  DATA_MATRIX(X);
  PARAMETER_VECTOR(beta1);
  PARAMETER_VECTOR(beta2);
  PARAMETER_VECTOR(beta3);
  PARAMETER_VECTOR(beta4);
  PARAMETER_VECTOR(beta5);
  PARAMETER_MATRIX(U);
  PARAMETER_VECTOR(rho);
  PARAMETER_VECTOR(sigma);
  PARAMETER_VECTOR(phi);
  Type pi = 3.141593;

  // Preparing
  vector<Type> rho_temp(10);
  rho_temp = rho;
  
  vector<Type> sigma_temp(5);
  sigma_temp = sigma;
  
  vector<Type> mu1(Y1.size());
  vector<Type> mu2(Y2.size());
  vector<Type> mu3(Y3.size());
  vector<Type> mu4(Y4.size());
  vector<Type> mu5(Y5.size());
  
  vector<Type> phi_temp(5);
  phi_temp = phi;
  
  // Linear predictor for mean model
  mu1 = exp(X*beta1 + U.col(0).array())/(1 + exp(X*beta1 + U.col(0).array()));
  mu2 = exp(X*beta2 + U.col(1).array())/(1 + exp(X*beta2 + U.col(1).array())); 
  mu3 = exp(X*beta3 + U.col(2).array())/(1 + exp(X*beta3 + U.col(2).array())); 
  mu4 = exp(X*beta4 + U.col(3).array())/(1 + exp(X*beta4 + U.col(3).array())); 
  mu5 = exp(X*beta5 + U.col(4).array())/(1 + exp(X*beta5 + U.col(4).array())); 
  
  // Full log-likelihood  
  Type nll1 = 0;
  for(int i=0; i<Y1.size(); i++)
    nll1 -= -0.5*log(2*pi)-0.5*log(exp(phi_temp(0)))-(1.5)*log(Y1(i)*(1-Y1(i)))-(1/(2*exp(phi_temp(0))))*pow((Y1(i)-mu1(i)),2)/(Y1(i)*(1-Y1(i))*pow(mu1(i),2)*pow((1-mu1(i)),2));
  Type nll2 = 0;
  for(int i=0; i<Y2.size(); i++)
    nll2 -= -0.5*log(2*pi)-0.5*log(exp(phi_temp(1)))-(1.5)*log(Y2(i)*(1-Y2(i)))-(1/(2*exp(phi_temp(1))))*pow((Y2(i)-mu2(i)),2)/(Y2(i)*(1-Y2(i))*pow(mu2(i),2)*pow((1-mu2(i)),2));
  Type nll3 = 0;
  for(int i=0; i<Y3.size(); i++)
    nll3 -= -0.5*log(2*pi)-0.5*log(exp(phi_temp(2)))-(1.5)*log(Y3(i)*(1-Y3(i)))-(1/(2*exp(phi_temp(2))))*pow((Y3(i)-mu3(i)),2)/(Y3(i)*(1-Y3(i))*pow(mu3(i),2)*pow((1-mu3(i)),2));
  Type nll4 = 0;
  for(int i=0; i<Y4.size(); i++)
    nll4 -= -0.5*log(2*pi)-0.5*log(exp(phi_temp(3)))-(1.5)*log(Y4(i)*(1-Y4(i)))-(1/(2*exp(phi_temp(3))))*pow((Y4(i)-mu4(i)),2)/(Y4(i)*(1-Y4(i))*pow(mu4(i),2)*pow((1-mu4(i)),2));
  Type nll5 = 0;
  for(int i=0; i<Y5.size(); i++)
    nll5 -= -0.5*log(2*pi)-0.5*log(exp(phi_temp(4)))-(1.5)*log(Y5(i)*(1-Y5(i)))-(1/(2*exp(phi_temp(4))))*pow((Y5(i)-mu5(i)),2)/(Y5(i)*(1-Y5(i))*pow(mu5(i),2)*pow((1-mu5(i)),2));
  Type nll = 0;
  for(int i = 0; i < Y1.size(); i++)
    nll += VECSCALE(UNSTRUCTURED_CORR(rho_temp), sigma_temp)(U.row(i));
  
  matrix<Type> Cor(5,5);
  Cor = UNSTRUCTURED_CORR(rho_temp).cov();
  REPORT(Cor);
  REPORT(sigma_temp);
  REPORT(exp(phi));
  return nll1 + nll2 + nll3 + nll4 + nll5 + nll;
}
